package cmd

import (
	"context"
	"fmt"
	"github.com/gogf/gf/v2/frame/g"
	"github.com/gogf/gf/v2/net/ghttp"
	"github.com/gogf/gf/v2/net/goai"
	"github.com/gogf/gf/v2/os/gcmd"
	"github.com/gogf/gf/v2/os/gcron"
	"github.com/gogf/gf/v2/os/glog"
	"nodeMonitor/internal/consts"
	"nodeMonitor/internal/router"
	"nodeMonitor/internal/service"
	"nodeMonitor/internal/task"
	"nodeMonitor/internal/websocket"
	"time"
)

var (
	Main = gcmd.Command{
		Name:  "main",
		Usage: "main",
		Brief: "start http server",
		Func: func(ctx context.Context, parser *gcmd.Parser) (err error) {

			//启动服务
			websocket.StartWebSocket(ctx)

			s := g.Server()

			s.Group("/", func(group *ghttp.RouterGroup) {

				router.BindController(group)
			})

			// Custom enhance API document.
			enhanceOpenAPIDoc(s)

			s.Run()
			return nil
		},
	}
)

func enhanceOpenAPIDoc(s *ghttp.Server) {
	openapi := s.GetOpenApi()
	openapi.Config.CommonResponse = ghttp.DefaultHandlerResponse{}
	openapi.Config.CommonResponseDataField = `Data`

	// API description.
	openapi.Info = goai.Info{
		Title:       consts.OpenAPITitle,
		Description: consts.OpenAPIDescription,
		Contact: &goai.Contact{
			Name: "GoFrame",
			URL:  "https://goframe.org",
		},
	}
}

func startPingStatus(ctx context.Context) error {
	taskStatusTime, err := g.Cfg().Get(ctx, "node.taskStatusTime")
	if err != nil {
		glog.Error(ctx, err.Error())
		return err
	}

	taskName, err := g.Cfg().Get(ctx, "node.taskName")
	if err != nil {
		glog.Error(ctx, err.Error())
		return err
	}

	s := fmt.Sprintf("0 */%d * * * *", taskStatusTime.Int())
	t, err := task.GetCronNextTime(s, time.Now())
	if err != nil {
		glog.Error(ctx, err.Error())
		return err
	}
	glog.Info(ctx, "NextTime:", t, err)
	logMsg := "NextTime:" + t.String()
	service.TaskLog().Create(ctx, "gcron", logMsg)
	_, err = gcron.AddSingleton(ctx, s, func(ctx context.Context) {
		taskStatusTime, err := g.Cfg().Get(ctx, "node.taskStatusTime")
		if err != nil {
			glog.Error(ctx, err.Error())
			return
		}

		s := fmt.Sprintf("0 */%d * * * *", taskStatusTime.Int())
		t, err := task.GetCronNextTime(s, time.Now())
		if err != nil {
			glog.Error(ctx, err.Error())
			return
		}
		glog.Info(ctx, "NextTime:", t, err)
		logMsg := "PingStatus NextTime:" + t.String()
		service.TaskLog().Create(ctx, "gcron", logMsg)
		task.PingStatus(ctx, t)
	}, taskName.String())
	return nil
}
